tic
%% Plotting of Source Localization

%% Initialization
clc
clear
%close all
file = 'Haavard1.wav';
[yss,fs] = audioread(file);
ly = length(yss);

% Cutting out region of interest
[yx,fs] = audioread(file, [round(21.975*fs) round(22.025*fs)]);

% Bandpass filtering
wn = [700/(fs/2) 3000/(fs/2)];
[b,a] = butter(6,wn);
ys = filter(b,a,yx);

%% Element positions in array
c = 343; % Speed of sound
dt = 1/fs; % time step
pos; % Reading the positions of the array
% Uncomment in 'pos' if you want to plot the element positions

xpos = y; ypos = x; zpos = zeros(1, 64);
normal_az = zeros(1, 64); % Azimuth angle is zero for all mics
normal_el = zeros(1, 64); % Mics point out from the array
positions = [zpos; xpos; ypos];

%% Creating the customized microphone element

Mic = phased.OmnidirectionalMicrophoneElement('FrequencyRange',[0,fs/2]);

array = phased.ConformalArray('Element',Mic,'ElementPosition',positions,...
   'ElementNormal',[normal_az; normal_el]);


%% Plotting the orignal signal vs the collected signals

% Sweeping through different angles
azi = [-80 -70 -60 -50 -40 -30 -20 -10 0 10 20 30 40 50 60 70 80];
aziR = deg2rad(azi); 
matX = []; matZ = []; matY=[];
xx= [];yy = [];zz =[];

% Sweeping through different incident angles
for az = 0:length(azi)-1
    for el = 1:length(azi)
        
        InnVinkel = [azi(az+1); azi(el)];
        beamformer = phased.TimeDelayBeamformer('SensorArray',array,...
            'SampleRate',fs,'PropagationSpeed',c,...
            'Direction', InnVinkel);
        Y = beamformer(ys);
        
        % Finding the energy in the signal
        Ysum = sum(Y.^2);
        
        % Converting from spherical to cartesian coordinates
        [x,y,z] = sph2cart(aziR(az+1),aziR(el),Ysum);
        xx = [xx x]; yy = [yy y]; zz = [zz z];
        
    end
    
    % Saving the coordinates of all the incident angles
    matX = [matX; xx(az*length(azi)+1:(az+1)*length(azi))];
    matY = [matY; yy(az*length(azi)+1:(az+1)*length(azi))];
    matZ = [matZ; zz(az*length(azi)+1:(az+1)*length(azi))];
end

%% Plotting the 3D figure of the array response
figure()
mesh(matX, matY, matZ)
set(gca, 'xscal', 'log'); % In order to get dB values
xlabel('value'); ylabel('y axis'); zlabel('z axis');

%%
toc

