clc
clear

%% Initializing
load('cepstra_arrayTale05.mat'); % cepstra from speech [array]
load('GMModel125.mat'); % Speech model (made of pure speech)

%load('Ncepstra_array.mat'); % Noise cepstra
%load('NModel.mat'); % Noise model (pure noise)


model = GMModel;

[M,k] = size(model.mu); % M=512, k=12
N = length(cepstra); % Amount of cepstra components, 7825

% Obtaining mu, sigma, c and x
mu = model.mu; % 512x12
Sigma = model.Sigma; % 1x12x512
c = model.ComponentProportion; % 1x512
x = cepstra; % 7825x12

% Initializing
LLR = [];

% tilde_mu: Adding 1 at the end of each vector
muTInit = [mu,ones(M,1)]';

% Initializing the W matrix, with zeros at the end
W = [eye(k) zeros(k,1)];

%% Iterating through the algorithm
for iter = 1:100
   
    muT = W*muTInit; 

    % Initializing
    gam = zeros(M,N);
    psum = zeros(1,N);
    
    % Calculating denominator of gamma expression
    for m = 1:M
        %disp(m)
        prob = mvnpdf(x,muT(:,m)',Sigma(1,:,m))*c(m);
        gam(m,:) = prob';
        psum = psum + prob';
    end
    
     % Finding gamma
    for m = 1:M
        %disp(m);
        gam(m,:) = gam(m,:) ./ psum;
    end

    % Beregn LLH:
    LLH = sum(log(psum));
    
    % Saving LLH intro matrix
    LLR = [LLR; LLH];
    
    % Calculating overline_Gamma
    Gamma = sum(gam,2);
    
    % Calculating observation matrix
    Obs = gam*x;
    
    % Calculating G over all 12 rows (need augmented mu)
    for i = 1:k % 12
        
        % Initializing G and z
        G = zeros(k+1,k+1);
        z = zeros(k+1,1);
        
        % Summing over all 512 mixtures
        for m = 1:M 
            G = G + Gamma(m)*Sigma(1,i,m)^(-2)*muTInit(:,m)*muTInit(:,m)';
            z = z + Sigma(1,i,m)^(-2)*Obs(m,i)*muTInit(:,m);
        end

        % Finding ith row of the new W matrix
        W(i,:) = linsolve(G,z);
    end
    
    disp(iter);
end % iter

%%
% Plotting the log likelihood
figure()
plot(LLR);
title('Log likelihood for different iterations', 'Fontsize', 16);
xlabel('Iteration', 'FontSize', 14);

% Finding the maximum likelihood
[val, I] = max(LLR);

figure()
plot(I-2:I+2,LLR(I-2: I+2));
title('Maximum Likelihood', 'Fontsize', 16);
xlabel('Iteration', 'FontSize', 14);


