%% Initializing

%clear
clc
%close all;
tic
% Initializing
fs = 48000; % Sampling frequency of the training data

%% Creating Noise set 

cd /Users/martevadla/Documents/MATLAB/wav-files
NoiseSett = dir('*.wav');
lengthNSet = length(NoiseSett);
myNoiseSet = cell(1,lengthNSet);


% myTrainSet contains all the training data without silence
for i = 1:lengthNSet
    myNoiseSet{i} = audioread(NoiseSett(i).name);
    
    % Removing silence: removing frames with lower amplitude than 1e-4
    indices = find(abs(myNoiseSet{i}) > 1e-4);
    myNoiseSet{i} = myNoiseSet{i}(indices);
end

cd /Users/martevadla/Documents/MATLAB


%% Creating validation set with 40 speakers
% Each unique speaker with a unique sentence

cd /Users/martevadla/Documents/MATLAB/wav-files/Val
ValSett = dir('*.wav');
lengthVSet = length(ValSett);
myValSet = cell(1,lengthVSet);

% myValSet contains all the validation data without silence
for i = 1:lengthVSet
    myValSet{i} = audioread(ValSett(i).name);
    
    % Removing silence: removing frames with lower amplitude than 1e-4
    indices = find(abs(myValSet{i}) > 1e-4);
    myValSet{i} = myValSet{i}(indices);
end

cd /Users/martevadla/Documents/MATLAB

%% Resample down to 24kHz

for i = 1:lengthNSet
    y = myNoiseSet{i};
    ly = length(y);
    y = resample(y,1,2);
    myNoiseSet{i} = y;
end

for i = 1:lengthVSet
    y = myValSet{i};
    ly = length(y);
    y = resample(y,1,2);
    myValSet{i} = y;
end

Fs = round(fs*(length(y)/ly)); % 24kHz

%% MFCC step: Extracting the cepstra from the training set

Ncepstra = []; mattc0 = [];
for i = 1:lengthNSet
    Y = myNoiseSet{i};
    %{
    YL = length(Y);
    start_point = YL*rand();
    end_point = start_point + 0.1*Fs;
    while end_point > YL || round(start_point)==0
        start_point = YL*rand();
        end_point = start_point + 0.1*Fs;
    end
    Y = Y(round(start_point):round(end_point));
    %}
    Ncep = melfcc(Y, Fs);
    Ncep = Ncep';
    [N, ~] = size(Ncep);
    avrg = (1/N)*sum(Ncep);
    Ncep = Ncep - avrg;
    Ncepstra = [Ncepstra; Ncep]; 
    mattc0 = [mattc0; Ncepstra(:,1)];
end

Ncep_c0 = Ncepstra(:,1);
Ncepstra = Ncepstra(:,2:13);

%% Extracting cepstra of the validation set

VNcepstra = [];
for i = 1:lengthVSet
    Y = myValSet{i};
    %{
    YL = length(Y);
    start_point = YL*rand();
    end_point = start_point + 0.1*Fs;
    while end_point > YL || round(start_point)==0
        start_point = YL*rand();
        end_point = start_point + 0.1*Fs;
    end
    Y = Y(round(start_point):round(end_point));
    %}
    Vcep = melfcc(Y, Fs);
    Vcep = Vcep';
    [Nn, ~] = size(Vcep);
    avrg = (1/Nn)*sum(Vcep);
    Vcep = Vcep - avrg;
    VNcepstra = [VNcepstra; Vcep];
end

VNcep_c0 = VNcepstra(:,1);
VNcepstra = VNcepstra(:,2:13);

%% Fitting a Gaussian Mixture Model to the cepstra data

options = statset('TolFun',1e-4);
NModel = fitgmdist(Ncepstra, 2048, 'CovarianceType', 'diagonal',...
    'RegularizationValue',0.001*min(var(Ncepstra)),'Options',options);

%% Creating a density function from the model and the validation cepstra

N = pdf(NModel, VNcepstra); 
Nlog = log(N);

figure()
histogram(Nlog); title('Histogram of log Noise model');
toc