
%% Initializing
clear
clc
%close all;

Fs = 24000;
sta = -100;
stop = 100;
step = 20;
EER = []; Terskel = [];
LLH = [];

% Loading models
load('GMModel125.mat'); % Original model for speech
%load('new_model2.mat'); % Adapted model for speech with 0.5s updated

Gmodel = GMModel;

%load('A_NModel_1.mat'); % Adapted model for noise 1 mix
% load('A_NModel_2048.mat'); % Adapted model for noise 2048 mix

load('NModel2048.mat'); % Original model for noise, 2048 mixtures

Nmodel = NModel;

tale = 'array speech';
Noise = 'array noise';

%%

for num = 1:1

% Collecting test set
for what = [{tale}, {Noise}]
    if strcmp(what, 'pure speech')
        % Pure speech
        cd /Users/martevadla/Documents/MATLAB/Master/Testsett2
    elseif strcmp(what, 'array speech')
        % Array speech
        cd /Users/martevadla/Documents/MATLAB/Test/Array_tale
    elseif strcmp(what, 'pure noise')
        % Noise test set
        cd /Users/martevadla/Documents/MATLAB/wav-files/Test
    elseif strcmp(what, 'array noise')
        % Noise test set from array
        cd /Users/martevadla/Documents/MATLAB/Test/Array_N
    end
    
    testSett = dir('*.wav');
    lengthTSet = length(testSett);
    myTestSet = cell(1,lengthTSet);
    for i = 1:lengthTSet
        [myTestSet{i}, fsamp] = audioread(testSett(i).name);
    end
    
    cd /Users/martevadla/Documents/MATLAB
    
    % Downsampling
    if fsamp == 48000
        for i = 1:lengthTSet
            y = myTestSet{i};
            ly = length(y);
            y = resample(y,1,2);
            myTestSet{i} = y;
        end
    end
    
    
    % Test Sample
 
    Res = []; % cepstra = [];
    for T = sta:step:stop
        LLRa = [];
        speech = 0;
        noise = 0;
        
        % Going through every single test sample
        for i = 1:length(myTestSet)
            
            Y = myTestSet{i};
            cep = melfcc(Y, Fs);
            
            cep = cep';
            [M, ~] = size(cep);
            avrg = (1/M)*sum(cep);
            cep = cep - avrg;
            cep = cep(:,2:13);
            lcep = length(cep);
            
            % Extracting 25ms-frames from the recording
            if lcep >num
                Sample = cep(round(lcep/2)-floor(num/2):round(lcep/2)+(floor((num-1)/2)), :);
                % cep = cepstra of file
                % lcep = length of cepstra of file
                % num = number of frames to be extracted
            else
                Sample=cep;
            end
            
            %% Creating a pdf for the speech model
            
            P = pdf(Gmodel, Sample);
            Plog = log(P);
            
            %% Creating a pdf for the noise model
            
            N = pdf(Nmodel, Sample);
            Nlog = log(N);
            
            %% Determining threshold
            
            LLR = sum(Plog)-sum(Nlog); % Log likelihood ratio
            
            LLH = [LLH; LLR];
            %% Determining whether speech or noise
            
            if round(LLR) >= T
                speech = speech + 1; % fprintf('Speech \n');
            else
                noise = noise + 1;% fprintf('Noise \n');
            end
            a = round(LLR);
            LLRa = [LLRa a];
        end
        Res = [Res;speech, noise;];
    end
    %
    Tot = Res(1,1) + Res(1,2);
    
    if strcmp(what, Noise)
        FA = 100*Res(:,1)/Tot; % False Acceptance for all speech
        TR = 100*Res(:,2)/Tot; % Correct Rejection for all noise
    elseif strcmp(what, tale)
        TA = 100*Res(:,1)/Tot; % Correct Acceptance
        FR = 100*Res(:,2)/Tot; % False Rejection for all noise
    end
end

% Plotting
t = linspace(sta,stop,length(FA))';

figure(); 
hold on;
plot(t, FR, '-b', 'LineWidth', 2);
plot(t, FA, '--b', 'LineWidth', 2);
grid on;
xlabel('Threshold [T]', 'Fontsize', 14);
ylabel('Percentage of errors [%]', 'Fontsize', 14);
title('Detection Error Tradeoff Curve','Fontsize', 16);


 L1 = [t, FA]'; L2 = [t, FR]';
 Inter = InterX(L1,L2);
 
disp('--------------------------------------------');
fprintf('DET between %s and %s:  \n', tale, Noise);
disp('--------------------------------------------');
if isempty(Inter)
    disp('No intersection');
    legend({'FR', 'FA'}, 'Fontsize', 14);
else
    plot(Inter(1), Inter(2), '*r', 'LineWidth', 5);
    fprintf('Optimal Threshold: T = %2.2f\n', Inter(1));
    fprintf('EER intersection at %2.2f%%\n\n', Inter(2));
    legend({'FR', 'FA', 'EER Intersection'}, 'Fontsize', 14);
end

EER = [EER; Inter(2)];
Terskel = [Terskel;Inter(1)];
disp(num)

end


