%% Initializing
tic
%clear
clc
%close all;

% Loading the name, total duration, start and end of each sentence
load('TrainStruct.mat');
load('ValStruct.mat');

% Initializing
fs = 48000; % Sampling frequency of the training data

%% Creating training set with 125 different speakers
% Each unique speaker with a unique sentence

% cd /Users/martevadla/Documents/MATLAB/Master/Treningssett
cd /Users/martevadla/Documents/MATLAB/Test/Array_tale
trainSett = dir('*.wav');
lengthSet = length(trainSett);
myTrainSet = cell(1,lengthSet);
%%
% myTrainSet contains all the training data without silence
for i = 1:lengthSet
    [myTrainSet{i}, FS] = audioread(trainSett(i).name);
    lenFil = length(myTrainSet{i}); % length in samples
    id_file = getfield(trainSett,{i,1},'name'); % med .wav
    
    if FS < 48000
        disp(id_file);
    end
    % Cutting silence at start and end for 48kHz
    SpeechStart = round(TrainStruct(i).info.Start *fs);
    SpeechEnd = round(TrainStruct(i).info.End *fs);
    
    myTrainSet{i} = myTrainSet{i}(SpeechStart:SpeechEnd);
    %audiowrite(trainSett(i).name,myTrainSet{i},FS);
end

cd /Users/martevadla/Documents/MATLAB

%% Creating validation set with 35 speakers
% Each unique speaker with a unique sentence

cd /Users/martevadla/Documents/MATLAB/Master/Valideringssett2
ValSett = dir('*.wav');
lengthVSet = length(ValSett);
myValSet = cell(1,lengthVSet);

% myValSet contains all the validation data without silence
for i = 1:lengthVSet
    myValSet{i} = audioread(ValSett(i).name);
    lenFil = length(myValSet{i}); % length in samples
    id_file = getfield(ValSett,{i,1},'name'); % med .wav
    
    % Cutting silence at start and end for 48kHz
    SpeechStart = round(ValStruct(i).info.Start *fs);
    SpeechEnd = round(ValStruct(i).info.End *fs);
    
    myValSet{i} = myValSet{i}(SpeechStart:SpeechEnd); 
end


%% Resample down to 24kHz

% sound(myTrainSet{78}, 48000);
for i = 1:lengthSet
    y = myTrainSet{i};
    ly = length(y);
    y = resample(y,1,2);
    myTrainSet{i} = y;
end

for j = 1:lengthVSet
    x = myValSet{j};
    lx = length(x);
    x = resample(x,1,2);
    myValSet{j} = x;
end

% sound(myTrainSet{78}, 24000);
Fs = round(fs*(length(y)/ly)); % 24kHz
cd /Users/martevadla/Documents/MATLAB

%% MFCC step: Extracting the cepstra from the training set

cepstra = []; matt = []; mattc0 = [];
for i = 1:lengthSet
    Y = myTrainSet{i};
    cep = melfcc(Y, Fs);
    cep = cep';
    [N, ~] = size(cep);
    
    % Cepstral Mean Normalization
    avrg = (1/N)*sum(cep);
    cep = cep - avrg;
    
    % Saving the MFCCs in a matrix
    cepstra = [cepstra; cep]; 
end

cepstra = cepstra(:,2:13);

%% Extracting cepstra of the validation set

Vcepstra = [];
for i = 1:lengthVSet
    Y = myValSet{i};     
    Vcep = melfcc(Y, Fs);
    Vcep = Vcep';
    [N, ~] = size(Vcep);
    avrg = (1/N)*sum(Vcep);
    Vcep = Vcep - avrg;
    Vcepstra = [Vcepstra; Vcep];
end

Vcep_c0 = Vcepstra(:,1);
Vcepstra = Vcepstra(:,2:13);

%% Fitting a Gaussian Mixture Model to the cepstra data

options = statset('TolFun',1e-4);
GMModel = fitgmdist(cepstra, 512, 'CovarianceType', 'diagonal',...
    'RegularizationValue',0.001*min(var(cepstra)),'Options',options);

%% Creating a density function from the model and the validation cepstra

P = pdf(GMModel, Vcepstra); 


Plog = log(P);

%% Creating a universal background model for non-speech

my = mean(cepstra);% size 1x13
Covariance = 5*cov(cepstra); % size 13x13

R2 = mvnrnd(my,Covariance,length(Vcepstra));

Nmodel = fitgmdist(R2, 1, 'CovarianceType', 'diagonal', 'RegularizationValue'...
    ,0.001*min(var(cepstra)), 'Options', options);
Nmod = pdf(Nmodel, R2); Nmodlog = log(Nmod);

%%
figure()
histogram(Plog); hold on;
histogram(Nmodlog); hold off;
title('Probability density functions for the two models', 'fontsize', 16);
legend({'Speech model', 'Noise model'}, 'Fontsize', 14, 'Location', 'NorthWest');



